/************************************************************************
 *   Solid Internet Relay Chat Daemon, src/m_shun.c
 *   Copyright (C) 2004 Juan Baez
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* $Id: m_shun.c,v 1.1.2.3 2004/07/30 01:31:19 tux316 Exp $ */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "userban.h"
#include "h.h"
#if defined( HAVE_STRING_H )
#include <string.h>
#else
#include <strings.h>
#endif

extern char *smalldate(time_t);	/* defined in s_misc.c */
extern int  myisnumber(char *);
extern char *mycluster(char *hostname);

int m_shun(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    struct userBan *ban, *oban;
    char        buffer[1024];
    char       *user, *host;
    char       *reason;
    char       *current_date;
    aClient    *acptr;
    char        tempuser[USERLEN + 2];
    char        temphost[HOSTLEN + 1];
    int         temporary_shun_time = 0;	/* -Dianora */
    time_t      temporary_shun_time_seconds = 0;
    int         time_specified = 0;
    char       *argv;
    int         i;
    char       fbuf[512];

    if (!IsAnOper(sptr) && !IsServer(sptr)) 
    {
		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
		return 0;
    }

    if (parc < 2) 
    {
		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS),
			me.name, parv[0], "SHUN");
		return 0;
    }

    argv = parv[1];

    if ((temporary_shun_time = myisnumber(argv)) >= 0) 
    {
		if (parc < 3) 
		{
			sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS),
			   me.name, parv[0], "SHUN");
			return 0;
		}
		if (temporary_shun_time > 1080000)
			temporary_shun_time = 1080000;
		temporary_shun_time_seconds = (time_t) temporary_shun_time;

		argv = parv[2];
		parc--;
		time_specified = 1;
    }
    else
    {
		temporary_shun_time = 0; 
    }
    
    if(strchr(argv, ' '))
    {
		sendto_one(sptr,
			":%s NOTICE %s :Poorly formatted hostname "
			"(contains spaces). Be sure you are using the form: "
			"/quote SHUN [time] <user@host/nick> :<reason>",
		me.name, parv[0]);
		return 0;
    }
    if ((host = strchr(argv, '@')) || *argv == '*') 
    {
		if (host) 		
		{
			user = argv;	
			*(host++) = '\0';
		}
		else 
		{
			user = "*";	
			host = argv;
		}
		if (!*host)
			host = "*";
		strncpyzt(tempuser, user, USERLEN + 2);
		strncpyzt(temphost, host, HOSTLEN);
		user = tempuser;
		host = temphost;
    }
    else 
    {
	
		if (!(acptr = find_chasing(sptr, argv, NULL)))
			return 0;
		if (!acptr->user)
			return 0;

		if (IsServer(acptr)) 
		{
			sendto_one(sptr, ":%s NOTICE %s :Can't SHUN a server, use @'s "
				"where appropriate", me.name, parv[0]);
			return 0;
		}
		tempuser[0] = '*';
		if (*acptr->user->username == '~')
			strcpy(tempuser + 1, (char *) acptr->user->username + 1);
		else
			strcpy(tempuser + 1, acptr->user->username);
		user = tempuser;
		host = mycluster(acptr->user->host);
    }

    if (time_specified)
		argv = parv[3];
    else
		argv = parv[2];

#ifdef DEFAULT_KLINE_TIME
    if (time_specified <= 0)
    {
		temporary_shun_time = 30;
		temporary_shun_time_seconds = (time_t) temporary_shun_time *(time_t) 60;
    }
#endif
    if (parc > 2) 
    {
		if (*argv)
			reason = argv;
		else
			reason = "No reason";
    }
    else
		reason = "No reason";

    if (!match(user, "akjhfkahfasfjd") && !match(host, "ldksjfl.kss...kdjfd.jfklsjf")) 
    {
		sendto_one(sptr, ":%s NOTICE %s :Can't Shun *@*", me.name, parv[0]);
		return 0;
    }

    if (temporary_shun_time == 0 && strchr(reason, ':')) 
    {
		sendto_one(sptr, ":%s NOTICE %s :Invalid character ':' in comment", me.name, parv[0]);
		return 0;
    }

    if (temporary_shun_time == 0 && strchr(reason, '#')) 
    {
		sendto_one(sptr,":%s NOTICE %s :Invalid character '#' in comment",
		me.name, parv[0]);
		return 0;
    }

    ban = make_hostbased_ban(user, host);
    if(!ban)
    {
		sendto_one(sptr, ":%s NOTICE %s :Malformed ban %s@%s", me.name, parv[0],
			user, host);
		return 0;
    }
    if ((oban = find_userban_exact(ban, 0)))
    {
		if (oban->flags & UBAN_FREEZE) {
			if (MyClient(sptr))
			{
				sendto_one(sptr, ":%s NOTICE %s :[%s@%s] already %s for %s",
					me.name, parv[0], user, host, NETWORK_SHUNNED_NAME, 
					oban->reason ? oban->reason : "<No Reason>");
			}
			else
			{
				host = get_userban_host(ban, fbuf, 512);
				sendto_serv_butone(sptr, ":%s SHUN %d %s@%s :%s",
					parv[0], temporary_shun_time_seconds, user, host, reason);
			}
			userban_free(ban);
			return 0;
		}
    }

    current_date = smalldate((time_t) 0);

	if (IsServer(sptr))
		ircsprintf(buffer, "%s", reason);
	else
		ircsprintf(buffer, "%s (%s)", reason, current_date);

    ban->flags |= temporary_shun_time ? (UBAN_FREEZE|UBAN_TEMPORARY) : (UBAN_FREEZE);
    ban->reason = (char *) MyMalloc(strlen(buffer) + 1);
    strcpy(ban->reason, buffer);
    
    ban->timeset = timeofday;
    ban->duration = temporary_shun_time_seconds;

    if(user_match_ban(sptr, ban))
    {
		sendto_one(sptr, ":%s NOTICE %s :You attempted to add a shun [%s@%s] "
						 "which would affect yourself. Aborted.",
						 me.name, parv[0], user, host);
		userban_free(ban);
		return 0;
    }

    add_hostbased_userban(ban);

    /* Check local users against it */
    for (i = 0; i <= highest_fd; i++)
    {
        if (!(acptr = local[i]) || IsMe(acptr) || IsLog(acptr))
            continue;

        if (IsPerson(acptr) && user_match_ban(acptr, ban) && !IsShunned(acptr))
        {
            sendto_ops(NETWORK_SHUN_NAME" active for %s",
                       get_client_name(acptr, FALSE));

			sendto_serv_butone(sptr, ":%s GLOBOPS :%s active for %s", 
				NETWORK_SHUN_NAME, me.name, get_client_name(acptr, FALSE));

            SetShunned(acptr);
            i--;
        }
    }

    host = get_userban_host(ban, fbuf, 512);

    if(temporary_shun_time)
    {	

		sendto_realops("%s added a temporary %d sec(s) "NETWORK_SHUN_NAME" for [%s@%s] [%s]",
			parv[0], temporary_shun_time, user, host, reason);

		sendto_serv_butone(sptr, ":%s SHUN %d %s@%s :%s",
			parv[0], temporary_shun_time_seconds, user, host, reason);

		return 0;
    }
    sendto_serv_butone(sptr, ":%s SHUN %d %s@%s :%s",
		parv[0], temporary_shun_time_seconds, user, host, reason);
    sendto_realops("%s added a permanent shun for [%s@%s] [%s]",
		   parv[0], user, host, reason);
    return 0;
}

int m_unshun(aClient *cptr, aClient *sptr, int parc, char *parv[]) 
{
    struct userBan *ban;
    char *user, *host;
    int i;
    aClient *acptr;
    
    if (!IsAnOper(sptr))
    {
	sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
	return 0;
    }
	
    if(parc<2) 
    {
	sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0],"UNSHUN");
	return 0;
    }

    if ((host = strchr(parv[1], '@')) || *parv[1] == '*') {
	if (host) {
		user = parv[1];
		*(host++) = '\0';
	} else {
		user = "*";
		host = parv[1];
		}
	} else {
		sendto_one(sptr, ":%s NOTICE %s :Invalid parameters",me.name, parv[0]);
		return 0;
	}
	if ((user[0] == '*') && (user[1] == '\0') && (host[0] == '*') && (host[1] == '\0'))  {
		sendto_one(sptr, ":%s NOTICE %s :Cannot UNSHUN everyone",me.name, parv[0]);
		return 0;
    }

    ban = make_hostbased_ban(user, host);
	if(ban) {
		struct userBan *oban;
		ban->flags |= (UBAN_FREEZE);

		if((oban = find_userban_exact(ban, UBAN_FREEZE|UBAN_TEMPORARY))) {
			char tmp[512];
			host = get_userban_host(oban, tmp, 512);
			
			if (MyClient(sptr)) 
				sendto_one(sptr, ":%s NOTICE %s :Shun for [%s@%s] is removed",
					me.name, parv[0], user, host);

			sendto_ops("%s has removed the shun for: [%s@%s] (%d matches)",
				parv[0], user, host, 1);
			
			for (i = 0; i <= highest_fd; i++) {
				if (!(acptr = local[i]) || IsMe(acptr) || IsLog(acptr))
					continue;
				if (IsPerson(acptr) && user_match_ban(acptr, ban) && IsShunned(acptr)) {
					sendto_ops(NETWORK_SHUN_NAME" deleted for %s",
						get_client_name(acptr, FALSE));
					ClearShunned(acptr);
					i--;
				}
			}

			remove_userban(oban);
			userban_free(oban);
			userban_free(ban);

			sendto_serv_butone(cptr, ":%s UNSHUN %s@%s", sptr->name, user, host);
			return 0;
		}
		userban_free(ban);
	}
	if (MyClient(sptr)) 
		sendto_one(sptr, ":%s NOTICE %s :No shun for %s@%s was found",
	           me.name, parv[0], user, host);
    return 0;
}
